@@ -665,7 +665,9 @@ static void _do_parse_libs (Package *pkg, int argc, char **argv)
           flag->arg = g_strconcat (L_flag, p, NULL);
           pkg->libs = g_list_prepend (pkg->libs, flag);
 	}
-      else if (strcmp("-framework",p) == 0 && i+1 < argc)
+      else if ((strcmp("-framework", p) == 0 ||
+                strcmp("-Wl,-framework", p) == 0) &&
+               i+1 < argc)
         {
           /* Mac OS X has a -framework Foo which is really one option,
            * so we join those to avoid having -framework Foo
@@ -1079,9 +1081,10 @@ parse_line (Package *pkg, const char *untrimmed, const char *path,
 }
 
 Package*
-parse_package_file (const char *path, gboolean ignore_requires,
-		    gboolean ignore_private_libs,
-		    gboolean ignore_requires_private)
+parse_package_file (const char *key, const char *path,
+                    gboolean ignore_requires,
+                    gboolean ignore_private_libs,
+                    gboolean ignore_requires_private)
 {
   FILE *f;
   Package *pkg;
@@ -1101,6 +1104,7 @@ parse_package_file (const char *path, gboolean ignore_requires,
   debug_spew ("Parsing package file '%s'\n", path);
   
   pkg = g_new0 (Package, 1);
+  pkg->key = g_strdup (key);
 
   if (path)
     {
@@ -1141,3 +1145,39 @@ parse_package_file (const char *path, gboolean ignore_requires,
   
   return pkg;
 }
+
+/* Parse a package variable. When the value appears to be quoted,
+ * unquote it so it can be more easily used in a shell. Otherwise,
+ * return the raw value.
+ */
+char *
+parse_package_variable (Package *pkg, const char *variable)
+{
+  char *value;
+  char *unquoted;
+  GError *error = NULL;
+
+  value = package_get_var (pkg, variable);
+  if (!value)
+    return NULL;
+
+  if (*value != '"' && *value != '\'')
+    /* Not quoted, return raw value */
+    return value;
+
+  /* Maybe too naive, but assume a fully quoted variable */
+  unquoted = g_shell_unquote (value, &error);
+  if (unquoted)
+    {
+      g_free (value);
+      return unquoted;
+    }
+  else
+    {
+      /* Note the issue, but just return the raw value */
+      debug_spew ("Couldn't unquote value of \"%s\": %s\n",
+                  variable, error ? error->message : "unknown");
+      g_clear_error (&error);
+      return value;
+    }
+}
